@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -280,7 +280,7 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	                                                      0.0,
 	                                                      1.0,
 	                                                      0.0,
-	                                                      G_PARAM_READWRITE));
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	g_type_class_add_private (object_class, sizeof (TrackerMinerPrivate));
 
@@ -418,7 +418,7 @@ miner_finalize (GObject *object)
 	g_hash_table_unref (miner->private->pauses);
 	g_ptr_array_free (miner->private->async_calls, TRUE);
 
-	tracker_miner_dbus_shutdown (miner);
+	_tracker_miner_dbus_shutdown (miner);
 
 	G_OBJECT_CLASS (tracker_miner_parent_class)->finalize (object);
 }
@@ -428,9 +428,9 @@ miner_constructed (GObject *object)
 {
 	TrackerMiner *miner = TRACKER_MINER (object);
 
-	tracker_miner_dbus_init (miner, &dbus_glib_tracker_miner_object_info);
-	tracker_miner_dbus_add_name_watch (miner, "org.freedesktop.Tracker1",
-                                       store_name_monitor_cb);
+	_tracker_miner_dbus_init (miner, &dbus_glib__tracker_miner_object_info);
+	_tracker_miner_dbus_add_name_watch (miner, "org.freedesktop.Tracker1",
+                                            store_name_monitor_cb);
 }
 
 static void
@@ -440,7 +440,8 @@ store_name_monitor_cb (TrackerMiner *miner,
 {
 	GError *error = NULL;
 
-	g_debug ("Tracker-store availability has changed to %d", available);
+	g_debug ("Store availability has changed to %s", 
+		 available ? "AVAILABLE" : "UNAVAILABLE");
 
 	if (available && miner->private->availability_cookie != 0) {
 		tracker_miner_resume (miner,
@@ -448,7 +449,7 @@ store_name_monitor_cb (TrackerMiner *miner,
 		                      &error);
 
 		if (error) {
-			g_warning ("Error happened resuming miner: %s\n", error->message);
+			g_warning ("Error happened resuming miner, %s", error->message);
 			g_error_free (error);
 		}
 
@@ -461,7 +462,7 @@ store_name_monitor_cb (TrackerMiner *miner,
 		                                 &error);
 
 		if (error) {
-			g_warning ("Could not pause: %s", error->message);
+			g_warning ("Could not pause, %s", error->message);
 			g_error_free (error);
 		} else {
 			miner->private->availability_cookie = cookie_id;
@@ -671,12 +672,7 @@ static void
 sparql_cancelled_cb (GCancellable  *cancellable,
                      AsyncCallData *data)
 {
-	TrackerMinerPrivate *priv;
-
 	async_call_data_notify_error (data, 0, "SPARQL operation was cancelled");
-
-	priv = TRACKER_MINER_GET_PRIVATE (data->miner);
-
 	async_call_data_destroy (data, TRUE);
 }
 
@@ -784,14 +780,12 @@ tracker_miner_execute_update (TrackerMiner        *miner,
                               GAsyncReadyCallback  callback,
                               gpointer             user_data)
 {
-	TrackerMinerPrivate *priv;
 	AsyncCallData *data;
 
 	g_return_if_fail (TRACKER_IS_MINER (miner));
 	g_return_if_fail (sparql != NULL);
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-	priv = TRACKER_MINER_GET_PRIVATE (miner);
 	data = async_call_data_new (miner,
 	                            cancellable,
 	                            callback,
@@ -850,7 +844,6 @@ tracker_miner_execute_sparql (TrackerMiner        *miner,
                               GAsyncReadyCallback  callback,
                               gpointer             user_data)
 {
-	TrackerMinerPrivate *priv;
 	AsyncCallData *data;
 
 	g_return_if_fail (TRACKER_IS_MINER (miner));
@@ -858,7 +851,6 @@ tracker_miner_execute_sparql (TrackerMiner        *miner,
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (callback != NULL);
 
-	priv = TRACKER_MINER_GET_PRIVATE (miner);
 	data = async_call_data_new (miner, cancellable, callback, user_data, tracker_miner_execute_sparql);
 
 	data->id = tracker_resources_sparql_query_async (miner->private->client,
@@ -917,14 +909,12 @@ tracker_miner_execute_batch_update (TrackerMiner        *miner,
                                     GAsyncReadyCallback  callback,
                                     gpointer             user_data)
 {
-	TrackerMinerPrivate *priv;
 	AsyncCallData *data;
 
 	g_return_if_fail (TRACKER_IS_MINER (miner));
 	g_return_if_fail (sparql != NULL);
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-	priv = TRACKER_MINER_GET_PRIVATE (miner);
 	data = async_call_data_new (miner, cancellable, callback, user_data, tracker_miner_execute_batch_update);
 
 	data->id = tracker_resources_batch_sparql_update_async (miner->private->client,
@@ -976,13 +966,11 @@ tracker_miner_commit (TrackerMiner        *miner,
                       gpointer             user_data)
 
 {
-	TrackerMinerPrivate *priv;
 	AsyncCallData *data;
 
 	g_return_if_fail (TRACKER_IS_MINER (miner));
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-	priv = TRACKER_MINER_GET_PRIVATE (miner);
 	data = async_call_data_new (miner, cancellable, callback, user_data, tracker_miner_commit);
 
 	data->id = tracker_resources_batch_commit_async (miner->private->client,
@@ -1116,9 +1104,9 @@ tracker_miner_resume (TrackerMiner  *miner,
 
 /* DBus methods */
 void
-tracker_miner_dbus_get_status (TrackerMiner           *miner,
-                               DBusGMethodInvocation  *context,
-                               GError                **error)
+_tracker_miner_dbus_get_status (TrackerMiner           *miner,
+                                DBusGMethodInvocation  *context,
+                                GError                **error)
 {
 	guint request_id;
 
@@ -1133,9 +1121,9 @@ tracker_miner_dbus_get_status (TrackerMiner           *miner,
 }
 
 void
-tracker_miner_dbus_get_progress (TrackerMiner           *miner,
-                                 DBusGMethodInvocation  *context,
-                                 GError                **error)
+_tracker_miner_dbus_get_progress (TrackerMiner           *miner,
+                                  DBusGMethodInvocation  *context,
+                                  GError                **error)
 {
 	guint request_id;
 
@@ -1151,9 +1139,9 @@ tracker_miner_dbus_get_progress (TrackerMiner           *miner,
 }
 
 void
-tracker_miner_dbus_get_pause_details (TrackerMiner           *miner,
-                                      DBusGMethodInvocation  *context,
-                                      GError                **error)
+_tracker_miner_dbus_get_pause_details (TrackerMiner           *miner,
+                                       DBusGMethodInvocation  *context,
+                                       GError                **error)
 {
 	GSList *applications, *reasons;
 	GStrv applications_strv, reasons_strv;
@@ -1195,11 +1183,11 @@ tracker_miner_dbus_get_pause_details (TrackerMiner           *miner,
 }
 
 void
-tracker_miner_dbus_pause (TrackerMiner           *miner,
-                          const gchar            *application,
-                          const gchar            *reason,
-                          DBusGMethodInvocation  *context,
-                          GError                **error)
+_tracker_miner_dbus_pause (TrackerMiner           *miner,
+                           const gchar            *application,
+                           const gchar            *reason,
+                           DBusGMethodInvocation  *context,
+                           GError                **error)
 {
 	GError *local_error = NULL;
 	guint request_id;
@@ -1238,10 +1226,10 @@ tracker_miner_dbus_pause (TrackerMiner           *miner,
 }
 
 void
-tracker_miner_dbus_resume (TrackerMiner           *miner,
-                           gint                    cookie,
-                           DBusGMethodInvocation  *context,
-                           GError                **error)
+_tracker_miner_dbus_resume (TrackerMiner           *miner,
+                            gint                    cookie,
+                            DBusGMethodInvocation  *context,
+                            GError                **error)
 {
 	GError *local_error = NULL;
 	guint request_id;
@@ -1276,10 +1264,10 @@ tracker_miner_dbus_resume (TrackerMiner           *miner,
 }
 
 void
-tracker_miner_dbus_ignore_next_update (TrackerMiner           *miner,
-                                       const GStrv             urls,
-                                       DBusGMethodInvocation  *context,
-                                       GError                **error)
+_tracker_miner_dbus_ignore_next_update (TrackerMiner           *miner,
+                                        const GStrv             urls,
+                                        DBusGMethodInvocation  *context,
+                                        GError                **error)
 {
 	guint request_id;
 
